/* snd2voc.c */

/*
   Converts raw mac soundfiles (.SOU) and soundtool files (.SND)
   to Soundblaster .VOC format.  You need to specify the sample
   rate with raw (.SOU) files.  It gets the rate from the header
   of the .SND files, so you shouldn't specify it.
*/


/*
    I, Bailey Brown, hereby release this code to the public domain.
    If you find any bugs, please let me know on compuserve
    at 73020,3442 (I'm also on Genie, but I've never used Genie mail).
*/

#include <io.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

#define BUFSIZE     (8*1024)

#define HEADERSIZE  (0x001a)

#define MAGICSTR    "Creative Voice File"
#define EOFCHAR     (0x1a)
#define DATAOFFSET  (0x001a)   /* word offset to start of data */
#define VERSION     (0x010a)   /* version number */
#define VERSIONC    (0x1129)   /* 2's comp of version no. + 1234h */
#define TYPE        (1)
#define VOCTERM     (0)

#define COMP8BIT    (0)

#define SRBYTE( sample_rate )  (256 - 1000000/(sample_rate))

main(int argc, char *argv[])
{
    long size;
    unsigned short rate = 0;

    char comptype = COMP8BIT;

    unsigned char header[HEADERSIZE+6];
    unsigned char *ph;
    unsigned char *s;
    unsigned char ch;

    int ifd, ofd;
    int bytesread;

    unsigned char buf[BUFSIZE];

    char ifname[128];
    char ofname[128];

    int has_ext;


    if (argc < 3) {
        fprintf(stderr, "usage: snd2voc soundfile vocfile [sampling rate]\n");
        exit(1);
    }
    if (argc > 3)
        rate = atoi(argv[3]);


    ph = header;
    s = MAGICSTR;

    while (*s)
        *ph++ = *s++;

    *ph++ = EOFCHAR;

    *((int*)ph)++ = DATAOFFSET;
    *((int*)ph)++ = VERSION;
    *((int*)ph)++ = VERSIONC;

    *ph++ = TYPE;

    strcpy(ifname, argv[1]);
    strcpy(ofname, argv[2]);

    ifd = open(ifname, O_RDONLY | O_BINARY);
    if (ifd < 0) {
        strcat(ifname, ".snd");
        ifd = open(ifname, O_RDONLY | O_BINARY);
    }
    if (ifd < 0) {
        strcpy(ifname, argv[1]);
        strcat(ifname, ".sou");
        ifd = open(ifname, O_RDONLY | O_BINARY);
    }
    if (ifd < 0) {
        fprintf(stderr, "can't open input file: %s\n", argv[1]);
        exit(1);
    }


    s = ofname;
    has_ext = 0;
    while (*s && !has_ext) {
        if (*s++ == '.') {
            has_ext = 1;
        }
    }
    if (!has_ext) {
        strcat(ofname, ".voc");
    }

    ofd = open(ofname, O_WRONLY | O_CREAT | O_BINARY);
    if (ofd < 0) {
        fprintf(stderr, "can't open output file: %s\n", argv[2]);
        exit(1);
    }

    s = ifname;
    while( *s && *s != '.') s++;

    if (*(s+2) == 'n' || *(s+2) == 'N') {
        read(ifd, &rate, sizeof(unsigned short));
        read(ifd, &rate, sizeof(unsigned short));
        lseek(ifd, 4, SEEK_CUR);
        size = filelength(ifd) + 2 - 4*2;
    } else {
        size = filelength(ifd) + 2;
        if (rate == 0) {
            fprintf(stderr, "must specify sampling rate\n");
            exit(1);
        }
    }


    *ph++ = size & 0x0000ff;
    *ph++ = (size & 0x00ff00) >> 8;
    *ph++ = (size & 0xff0000) >> 16;

    *ph++ = SRBYTE(rate);
    *ph++ = comptype;


    write(ofd, &header, sizeof(header));

    while ((bytesread = read(ifd, &buf, BUFSIZE)) > 0) {
        write(ofd, &buf, bytesread);
    }

    ch = VOCTERM;
    write(ofd, &ch, 1);

    close(ifd);
    close(ofd);

    return 0;
}











